@ *************************************************************

  PROGRAM TO ESTIMATE ENGEL CURVE OF FIGURE 2.

**************************************************************@

new;
library pgraph;
graphset;
format 8,4;

rparm = 1.e-9;  @ ***** Stabilization Parameter ***** @
nbase = 4;
ksp = 3;  @ ***** Order of the spline ***** @
nk0 = 4; @ ***** No. of spline knots in [-1,1] ***** @

@ ***** READ DATA ***** @

 dataset = "c:\\gauss50\\npiv\\Fisher-Schultz\\EMA\\Fig2";
 open fy=^dataset;
 n = rowsf(fy);
 yz=readr(fy,n);
 close(fy);
 n = rows(yz);

@ ***** SELECT VARIABLES ***** @
 
 y = yz[.,1];
 x = yz[.,2];
 w = yz[.,3];

@ ***** Transform X and W to Unit Interval ***** @

delx = maxc(x) - minc(x);
minx = minc(x);
x = (x - minc(x))./(maxc(x) - minc(x));
w = (w - minc(w))./(maxc(w) - minc(w));

@ ***** ESTIMATE FOURIER COEFFICIENTS ***** @

   nk = nk0;
   xx = 5*spbase(x,nk,ksp,n);
   ww = 5*spbase(w,nk,ksp,n);
   qmat = xx'ww*ww'xx/(n^2);
   nmat = xx'ww*ww'/(n^2);
   rq = rows(qmat);
   qmat = qmat + rparm*eye(rq);
   bhat = invpd(qmat)*nmat*y;

yhat = xx*bhat;
xx = delx*x + minx;
tmp = xx~yhat;
tmp = sortc(tmp,1);
xy(tmp[.,1],tmp[.,2]); 


/************ END OF MAIN PROGRAM ********************************/


/************************************************************************/
/**** GENERATE n KNOT POINTS IN [0,1] FOR B-SPLINES OF ORDER k. ********/
/************************************************************************/
 proc(1) = kspl(nk1,k);

 local incr,mncr,nn1, kn0, strt, tmp0, tmp1;

 incr = 1/nk1;
 mncr = -incr;
 nn1 = nk1 + 1;
 kn0 = seqa(0,incr,nn1); 
 strt = -incr;
 tmp0 = rev(seqa(strt,mncr,k)); 
 strt = 1 + incr;
 tmp1 = seqa(strt,incr,k);
 retp(tmp0|kn0|tmp1);
 endp;

/************************************************************************/
/******** GENERATE MATRIX IDENTIFYING THE NON-ZERO TERMS OF THE *********/
/******** THE SPLINE SUMMAND AND THE PRODUCT COEFFICIENT FOR EACH *******/
/******** SPLINE COMPONENT.  THE P'TH COLUMN OF SMAT CORRESPONDS ********/
/******** TO THE P'TH BSPLINE. ******************************************/
/************************************************************************/
 proc(1) = spmat(k,xkt,del);

 local nk,np, smat,jj,p,i,jjm,imax,sgn,trm;

 nk = rows(xkt);
 np = nk - k - 1;
 
 smat = zeros(nk,np);
 jj = 1;
 jjm = nk;
 do until jj > jjm;
 p = 1;
 do until p > np;
 
 @ ***** COMPUTE THE PRODUCT THAT GIVES THE SIGN and FACTOR OF THE TERM **** @

    smat[jj,p] = (jj >= p)*(jj <= p + k + 1);
    i = p;
    imax = p + k + 1;
    sgn = 1;
    if smat[jj,p] > 0;
     do until i > imax;
       if i /= jj;
       sgn = sgn*del/(i - jj);
       endif;
       i = i + 1;
       endo;
     endif;
    
    smat[jj,p] = smat[jj,p]*sgn;
    p = p + 1;
    endo;
    jj = jj + 1;
    endo;
 retp(smat);
 endp;

/************************************************************************/
/***** EVALUATE P'TH BSPLINE AT THE VALUES OF X *************************/
/************************************************************************/

 proc(1) = bspl(x,p,k,xkt,smat);

 local term;
 
 term = ((abs(x' - xkt))^k).*(x' - xkt .> 0).*smat[.,p];

 retp(sumc(term));
 endp;

/**********************************************************************/
/******** EVALUATE THE SPLINE BASIS AT EACH VALUE OF X OR W ***********/
/**********************************************************************/

proc(1) = spbase(x,nk,k,n);

local nk1, del,xkt, smat,nkk,np,z, xs,ip;
  
 nk1 = nk - 1;
 del = nk1/2;

 xkt = kspl(nk1,k); 

 smat = spmat(k,xkt,del);
 nkk = rows(xkt);
 np = nkk - k - 1;

@ ***** EVALUATE SPLINE BASIS AT EACH X VALUE ***** @

 z = zeros(n,np);
 xs = zeros(n,np);
 ip = 1;
 do until ip > np;
  z[.,ip] = bspl(x,ip,k,xkt,smat);
  ip = ip + 1;
  endo;

@ ***** FORM "DESIGN MATRIX" OF SPLINE VALUES AT THE OBSERVED X'S ***** @

   xs = z;
retp(xs);
endp;  


