@ *************************************************************

  PROGRAM TO GENERATE FIG. 3 AND ESTIMATE AVERAGE DERIVATIVE

**************************************************************@

new;
library pgraph;
clear grid, bhat,invaht,nst,nsch,nboot,iquad,nbase,bhat,invaht,vars,delbar,cst,
    ngr;
graphset;
format 8,5;

rndseed 9858389; 

nboot = 500; @ ***** No. of Bootstrap Samples ***** @ 
ngr = 19;
nbase = 4;

dvar = 5;  @ ***** Index of dependent variable ***** @


@ ***** READ DATA ***** @

 dataset = "c:\\gauss50\\npiv\\Fisher-Schultz\\EMA\\Fig3";
 open fy=^dataset;
 n = rowsf(fy);
 yz=readr(fy,n);
 close(fy);

 svar = (yz[.,3] .< 45);
 yz = selif(yz,svar);
 n = rows(yz);

"Original size:  ";; n;

@ ***** GENERATE INTEGER VALUES for SCHOOL CODES ***** @

idx = 2;
wcode = 1;
do until idx > n;

  idy = idx - 1;
  if yz[idx,1] == yz[idy,1];
    wcode = wcode|wcode[idy];
    else;
    tmp = wcode[idy] + 1;
    wcode = wcode|tmp;
    endif;

idx = idx + 1;
endo;
nsch = maxc(wcode);


@ ***** SELECT VARIABLES ***** @
 
 y = yz[.,dvar];
 x = yz[.,3];
 w = yz[.,7];
 z = yz[.,6];

 vars = y~x~w~z~wcode;
 nst = rows(vars);
 cst = cols(vars);
 
output off;

@ ***** Transform X and W to Unit Interval ***** @

xmax = maxc(x);
xmin = minc(x);
wmax = maxc(w);
wmin = minc(w);
x = (x - xmin)./(xmax - xmin);
w = (w - wmin)./(wmax - wmin);

svar = (z .<= 3);
y = selif(y,svar);
x = selif(x,svar);
w = selif(w,svar);
n = rows(x);

xming = minc(x);
xmaxg = maxc(x);
del = (xmaxg - xming)/(ngr + 1);
x0 = xming + del;
grid = seqa(x0,del,ngr);
gr0 = grid*(xmax - xmin) + xmin;


@ ***** ESTIMATE FOURIER COEFFICIENTS ***** @

   xx = bfunc(x,nbase,3);
   ww = bfunc(w,nbase,3);
   xgrid = bfunc(grid,nbase,3);

   qmat = xx'ww*ww'xx/(n^2);
   nmat = xx'ww*ww'/(n^2);
   rq = rows(qmat); 
   bhat = invpd(qmat)*nmat*y;


yhat = xx*bhat;
dxx = zeros(n,1)~ones(n,1)~(2*xx[.,2])~(3*xx[.,3]);
dyhat = dxx*bhat;  @ ***** Derivative of ghat in transformed x units ***** @
dyhat = dyhat/(xmax - xmin);  @ ***** Derivative of ghat in original x units ***** @
xorig = (xmax - xmin)*x + xmin;
tmp = xorig~yhat;
svar = (xorig .>= 20).*(xorig .< 40);
dyhat = selif(dyhat,svar);

dyhat = meanc(dyhat);  @ ***** Average Derivative ***** @
"Average derivative:  ";; dyhat;

@ ***** USE BOOTSTRAP TO GET CONFIDENCE BAND and STANDARD ERROR OF AV DERIV ***** @

invaht = inv(ww'xx);
uhat = y - xx*bhat;

delt = xgrid*inv(ww'xx)*ww';
delt = (delt').*uhat;
sn = stdc(delt);
delbar = meanc(uhat.*ww);
delbar = xgrid*invaht*delbar;

{cval, sedx} = bootstr(bhat,invaht,delbar);

yhat = xgrid*bhat;
ylow = yhat - cval;
yhi = yhat + cval;
xorig = (xmax - xmin)*grid + xmin;
tmp = gr0~yhat~ylow~yhi;

tmp = sortc(tmp,1);

"Average derivative and std error:  ";; dyhat;; sedx;
"";
"x, yhat, ylow, yhi:  ";
tmp;

xtics(20,40,5,0);
ytics(72,84,2,0);
xy(tmp[.,1],tmp[.,2:4]); 




/************ END OF MAIN PROGRAM ********************************/

proc(2) = bootstr(bhat,invaht,delbar);

/*****************************************************************

PROCEDURE TO FIND BOOTSTRAP UNIFORM CONFIDENCE INTERVALS AND 
STANDARD ERROR OF AVERAGE DERIVATIVE ESTIMATE

******************************************************************/

local isamp,nsamp,vboot,temp,isch,svar,grid,xboot,yboot,wboot, zboot,
  xbmax,xbmin,wbmax,wbmin,uboot,nbt,bhbt,wwb,xxb,delbt,swig,swstar,
  snbt,tstat,maxt,iboot,clvl,crit,xorig,dxx,dyhat,aderiv;

clvl = 0.95;
clvl = clvl*nboot;

@ ***** START BOOTSTRAP ITERATIONS ***** @

maxt = zeros(nboot,1);
aderiv = zeros(nboot,1);

iboot = 1;
do until iboot > nboot;

@ ***** Draw the bootstrap sample ***** @

isamp = 1;
nsamp = 0;



do until nsamp > nst;
isch = ceil(nsch*rndu(1,1));
svar = (vars[.,cst] .== isch);

  if isamp == 1;
  vboot = selif(vars,svar);
  else;
  temp = selif(vars,svar);
  vboot = vboot|temp;
  endif;

 nsamp = rows(vboot);
 isamp = isamp + 1;
 endo; 


@ ***** create BOOTSTRAP VARIABLES ***** @

 yboot = vboot[.,1];
 xboot = vboot[.,2];
 wboot = vboot[.,3];
 zboot = vboot[.,4];
 
output off;

@ ***** Transform X and W to Unit Interval ***** @

xbmax = maxc(xboot);
xbmin = minc(xboot);
wbmax = maxc(wboot);
wbmin = minc(wboot);
xboot = (xboot - xbmin)./(xbmax - xbmin);
wboot = (wboot - wbmin)./(wbmax - wbmin);

svar = (zboot .<= 3);
yboot = selif(yboot,svar);
xboot = selif(xboot,svar);
wboot = selif(wboot,svar);
nbt = rows(xboot);

@ ***** ESTIMATE BOOTSTRAP FOURIER COEFFICIENTS ***** @

   xxb = bfunc(xboot,nbase,3);
   wwb = bfunc(wboot,nbase,3);
   bhbt = inv(wwb'xxb)*wwb'yboot;
 

uboot = yboot - xxb*bhat;
delbt = wwb'uboot;   @ meanc(uboot.*wwb); @
swig = xgrid*invaht*delbt - delbar;


tstat = abs(swig);


maxt[iboot] = maxc(tstat);

@ ***** Average derivative calculation ***** @

dxx = zeros(nbt,1)~ones(nbt,1)~(2*xxb[.,2])~(3*xxb[.,3]);
xorig = (xbmax - xbmin)*xboot + xbmin;
svar = (xorig .>= 20).*(xorig .< 40);
dxx = selif(dxx,svar);
dxx = meanc(dxx);
dxx = dxx';

delbt = wwb'uboot;

dyhat = dxx*invaht*delbt; @ ***** Derivative of ghat in transformed x units ***** @
dyhat = dyhat/(xbmax - xbmin);  @ ***** Derivative of ghat in original x units ***** @



aderiv[iboot] = abs(dyhat);


iboot = iboot + 1;
endo;

maxt = sortc(maxt,1);
aderiv = sortc(aderiv,1);



retp(maxt[clvl],aderiv[clvl]);
endp;


 
proc bfunc(x,nf,ibase);

/******************************************************************

GENERATE THE BASIS FUNCTIONS.

******************************************************************/

local jvec,jv1,sq2, fj, z, rz, psi;

sq2 = sqrt(2);
jvec = seqa(1,1,nf);
z = 2*x - 1;


 rz = rows(z);
 psi = zeros(rz,10);
 psi[.,1] = ones(rz,1);
 psi[.,2] = (sqrt(3))*z;
 psi[.,3] = 0.5*(sqrt(5))*(3*(z^2) - 1);
 psi[.,4] = 0.5*(sqrt(7))*(5*(z^3)- 3*z);
 psi[.,5] = 0.125*3*(35*(z^4)-30*(z^2) + 3);
 psi[.,6] = 0.125*(sqrt(11))*(63*(z^5) - 70*(z^3) + 15*z);
 psi[.,7] = 0.0625*(sqrt(13))*(231*(z^6) - 315*(z^4) + 105*(z^2) - 5);
 psi[.,8] = 0.0625*(sqrt(15))*(429*(z^7) - 693*(z^5) + 315*(z^3) - 35*z);
 psi[.,9] = (1/128)*(sqrt(17))*(6435*(z^8) - 12012*(z^6) + 6930*(z^4) - 1260*(z^2) + 35);
 psi[.,10] = (1/128)*(sqrt(19))*(12155*(z^9) - 25740*(z^7) + 18018*(z^5) - 4620*(z^3) + 315*z);
 psi = psi[.,1:nf];


retp(psi);
endp;

